import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { unwrapResult } from '@reduxjs/toolkit';
import { shapesFixture } from '@/models/fixtures/shapesFixture';
import shapesReducer from './shapes.slice';
import { getShapes } from './shapes.thunks';
import { BioShapesState } from './shapes.types';

const mockedAxiosClient = mockNetworkNewAPIResponse();

const INITIAL_STATE: BioShapesState = {
  data: [],
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('shapes reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<BioShapesState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('shapes', shapesReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(shapesReducer(undefined, action)).toEqual(INITIAL_STATE);
  });

  it('should update store after succesfull getShapes query', async () => {
    mockedAxiosClient.onGet(apiPath.getShapes()).reply(HttpStatusCode.Ok, shapesFixture);

    const { type } = await store.dispatch(getShapes());
    const { data, loading, error } = store.getState().shapes;
    expect(type).toBe('vectorMap/getShapes/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(shapesFixture);
  });

  it('should update store after failed getShapes query', async () => {
    mockedAxiosClient.onGet(apiPath.getShapes()).reply(HttpStatusCode.NotFound, []);

    const action = await store.dispatch(getShapes());
    const { data, loading, error } = store.getState().shapes;

    expect(action.type).toBe('vectorMap/getShapes/rejected');
    expect(() => unwrapResult(action)).toThrow(
      "Failed to fetch shapes: The page you're looking for doesn't exist. Please verify the URL and try again.",
    );
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual([]);
  });

  it('should update store on loading getShapes query', async () => {
    mockedAxiosClient.onGet(apiPath.getShapes()).reply(HttpStatusCode.Ok, shapesFixture);

    const shapesPromise = store.dispatch(getShapes());

    const { data, loading } = store.getState().shapes;
    expect(data).toEqual([]);
    expect(loading).toEqual('pending');

    shapesPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } = store.getState().shapes;

      expect(dataPromiseFulfilled).toEqual(shapesFixture);
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
